import Vue from 'vue';
import Vuex from 'vuex';
import AxiosBootstrap from 'axios';
import * as _ from 'lodash/fp';
import router from '../router';

import * as base64 from 'base-64';
import * as utf8 from 'utf8';

Vue.use(Vuex);
const axios = AxiosBootstrap.create({
    baseURL: '/api',
});

axios.interceptors.response.use(response => response, error => {
    console.log('error interceptor fired');
    console.error(error); // todo: toast error
    console.log(Object.entries(error));

    if (error.isAxiosError) {
        const message = `
      <h3>A HTTP ${error.config.method} request failed.</h3>
      <p>
          url: ${error.config.url}
          <br>
          timeout: ${!!error.request.timeout}
          <br>
          response-body: ${error.response && error.response.body}
      </p>
    `;
        store.commit('createToast', {title: 'Error: HTTP', message, color: 'danger'});
    } else {
        store.commit('createToast', {title: 'Error: Unknown', message: error.toString(), color: 'danger'});
    }
    return Promise.reject(error);
});

const store = new Vuex.Store({
    state: {
        keyIncrement: 0,
        events: [],
        layout: 'cards',
        loadedItems: [],
        loadedBoxes: [],
        toasts: [],
        tickets: [],
        userRoles: ['admin', 'team', 'orga', 'user'],
        lastEvent: localStorage.getItem('lf_lastEvent') || '36C3',
        lastUsed: localStorage.getItem('lf_lastUsed') || {},
        remember: false,
        user: null,
        token: null,
        local_loaded: false,
    },
    getters: {
        getEventSlug: state => state.route && state.route.params.event ? state.route.params.event : state.lastEvent,
        getActiveView: state => state.route.name || 'items',
        getFilters: state => state.route.query,
        getBoxes: state => state.loadedBoxes,
        checkRole: state => role => state.userRoles.includes(role),
        isLoggedIn(state) {
            if (!state.local_loaded) {
                state.remember = localStorage.getItem('remember') === 'true'
                state.user = localStorage.getItem('user')
                state.token = localStorage.getItem('token')
                state.local_loaded = true
            }

            return state.user !== null && state.token !== null;
        },
    },
    mutations: {
        updateLastUsed(state, diff) {
            state.lastUsed = _.extend(state.lastUsed, diff);
            localStorage.setItem('lf_lastUsed', state.lastUsed);
        },
        updateLastEvent(state, slug) {
            state.lastEvent = slug;
            localStorage.setItem('lf_lastEvent', slug);
        },
        replaceEvents(state, events) {
            state.events = events;
        },
        changeView(state, {view, slug}) {
            router.push({path: `/${slug}/${view}`});
        },
        replaceLoadedItems(state, newItems) {
            state.loadedItems = newItems;
        },
        setLayout(state, layout) {
            state.layout = layout;
        },
        replaceBoxes(state, loadedBoxes) {
            state.loadedBoxes = loadedBoxes;
        },
        updateItem(state, updatedItem) {
            const item = state.loadedItems.filter(({uid}) => uid === updatedItem.uid)[0];
            Object.assign(item, updatedItem);
        },
        removeItem(state, item) {
            state.loadedItems = state.loadedItems.filter(it => it !== item);
        },
        appendItem(state, item) {
            state.loadedItems.push(item);
        },
        replaceTickets(state, tickets) {
            state.tickets = tickets;
        },
        updateTicket(state, updatedTicket) {
            const ticket = state.tickets.filter(({id}) => id === updatedTicket.id)[0];
            Object.assign(ticket, updatedTicket);
        },
        createToast(state, {title, message, color}) {
            var toast = {title, message, color, key: state.keyIncrement}
            state.toasts.push(toast);
            state.keyIncrement += 1;
            return toast;
        },
        removeToast(state, key) {
            state.toasts = state.toasts.filter(toast => toast.key !== key);
        },
        setRemember(state, remember) {
            state.remember = remember;
            localStorage.setItem('remember', remember);
        },
        setUser(state, user) {
            state.user = user;
            localStorage.setItem('user', user);
        },
        logout(state) {
            state.user = null;
            state.token = null;
            localStorage.removeItem('user');
            localStorage.removeItem('token');
            router.push('/login');
        },
    },
    actions: {
        async login({commit, dispatch, state}, {username, password, remember}) {
            commit('setRemember', remember);
            const data = await fetch('/api/2/auth/token/', {
                method: 'POST',
                headers: {'Content-Type': 'application/json'},
                body: JSON.stringify({username: username, password: password}),
                credentials: 'omit'
            }).then(r => r.json())
            if (data.token && data.key) {
                commit('setToken', data.token);
                commit('setUser', username);
                return true;
            } else {
                return false;
            }
        },
        async loadEvents({commit}) {
            const {data} = await axios.get('/1/events/');
            commit('replaceEvents', data);
        },
        changeEvent({dispatch, getters}, eventName) {
            router.push({path: `/${eventName.slug}/${getters.getActiveView}/`});
            dispatch('loadEventItems');
        },
        changeView({getters}, link) {
            router.push({path: `/${getters.getEventSlug}/${link.path}/`});
        },
        showBoxContent({getters}, box) {
            router.push({path: `/${getters.getEventSlug}/items/`, query: {box}});
        },
        async loadEventItems({commit, getters}) {
            const {data} = await axios.get(`/2/${getters.getEventSlug}/items/`);
            commit('replaceLoadedItems', data);
        },
        async searchEventItems({commit, getters}, query) {
            const foo = utf8.encode(query);
            const bar = base64.encode(foo);

            const {data} = await axios.get(`/2/${getters.getEventSlug}/items/${bar}/`);
            commit('replaceLoadedItems', data);
        },
        async loadBoxes({commit}) {
            const {data} = await axios.get('/2/boxes/');
            commit('replaceBoxes', data);
        },
        async updateItem({commit, getters}, item) {
            const {data} = await axios.put(`/2/${getters.getEventSlug}/item/${item.uid}/`, item);
            commit('updateItem', data);
        },
        async markItemReturned({commit, getters}, item) {
            await axios.put(`/2/${getters.getEventSlug}/item/${item.uid}/`, {returned: true});
            commit('removeItem', item);
        },
        async deleteItem({commit, getters}, item) {
            await axios.delete(`/2/${getters.getEventSlug}/item/${item.uid}/`, item);
            commit('removeItem', item);
        },
        async postItem({commit, getters}, item) {
            commit('updateLastUsed', {box: item.box, cid: item.cid});
            const {data} = await axios.post(`/2/${getters.getEventSlug}/item/`, item);
            commit('appendItem', data);
        },
        async loadTickets({commit}) {
            const {data} = await axios.get('/2/tickets/');
            commit('replaceTickets', data);
        },
        async sendMail({commit, dispatch}, {id, message}) {
            const {data} = await axios.post(`/2/tickets/${id}/reply/`, {message});
            await dispatch('loadTickets');
        }
    }
});

export default store;

store.dispatch('loadEvents').then(() => {
    store.dispatch('loadEventItems');
    store.dispatch('loadBoxes');
});
